@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2020 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2021 Snowflake Computing Inc. All right reserved.
 #
 
 import collections
@@ -79,8 +79,10 @@ from .tool.probe_connection import probe_connection
 from .vendored import requests
 from .vendored.requests.adapters import HTTPAdapter
 from .vendored.requests.auth import AuthBase
-from .vendored.requests.exceptions import ConnectionError, ConnectTimeout, ReadTimeout, SSLError
+from .vendored.requests.exceptions import ConnectionError, ConnectTimeout, InvalidProxyURL, ReadTimeout, SSLError
+from .vendored.requests.utils import prepend_scheme_if_needed, select_proxy
 from .vendored.urllib3.exceptions import ProtocolError, ReadTimeoutError
+from .vendored.urllib3.util.url import parse_url
 
 logger = logging.getLogger(__name__)
 
@@ -168,6 +170,32 @@ def is_retryable_http_code(code: int) -> bool:
     )
 
 
+class ProxySupportAdapter(HTTPAdapter):
+    """This Adapter creates proper headers for Proxy CONNECT messages."""
+
+    def get_connection(self, url, proxies=None):
+        proxy = select_proxy(url, proxies)
+        parsed_url = urlparse(url)
+
+        if proxy:
+            proxy = prepend_scheme_if_needed(proxy, 'http')
+            proxy_url = parse_url(proxy)
+            if not proxy_url.host:
+                raise InvalidProxyURL("Please check proxy URL. It is malformed"
+                                      " and could be missing the host.")
+            proxy_manager = self.proxy_manager_for(proxy)
+
+            # Add Host to proxy header SNOW-232777
+            proxy_manager.proxy_headers['Host'] = parsed_url.hostname
+            conn = proxy_manager.connection_from_url(url)
+        else:
+            # Only scheme should be lower case
+            url = parsed_url.geturl()
+            conn = self.poolmanager.connection_from_url(url)
+
+        return conn
+
+
 class RetryRequest(Exception):
     """Signal to retry request."""
     pass
@@ -533,6 +561,9 @@ class SnowflakeRestful(object):
                 return self._post_request(
                     url, headers, body, token=self.token, timeout=timeout)
 
+        if ret.get('data') and ret['data'].get('queryId'):
+            logger.debug('Query id: {}'.format(ret['data']['queryId']))
+
         if ret.get('code') == QUERY_IN_PROGRESS_ASYNC_CODE and _no_results:
             return ret
 
@@ -600,9 +631,11 @@ class SnowflakeRestful(object):
         parsed_url = urlparse(full_url)
         if not parsed_url.hostname.endswith(SNOWFLAKE_HOST_SUFFIX):
             return full_url
+        request_guid = str(uuid.uuid4())
         suffix = urlencode({
-            REQUEST_GUID: str(uuid.uuid4())
+            REQUEST_GUID: request_guid
         })
+        logger.debug(f'Request guid: {request_guid}')
         sep = '&' if parsed_url.query else '?'
         # url has query string already, just add fields
         return full_url + sep + suffix
@@ -890,8 +923,8 @@ class SnowflakeRestful(object):
 
     def make_requests_session(self):
         s = requests.Session()
-        s.mount('http://', HTTPAdapter(max_retries=REQUESTS_RETRY))
-        s.mount('https://', HTTPAdapter(max_retries=REQUESTS_RETRY))
+        s.mount('http://', ProxySupportAdapter(max_retries=REQUESTS_RETRY))
+        s.mount('https://', ProxySupportAdapter(max_retries=REQUESTS_RETRY))
         s._reuse_count = itertools.count()
         return s
 
